"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildGraphQLContext = void 0;
const lodash_1 = __importDefault(require("lodash"));
const graphql_1 = require("graphql");
const __1 = require("..");
const applyPluginsToSchemas_1 = __importDefault(require("./applyPluginsToSchemas"));
const applyPluginsToModels_1 = __importDefault(require("./applyPluginsToModels"));
const buildResolverContext_1 = __importDefault(require("./buildResolverContext"));
const buildPlugins_1 = __importDefault(require("./buildPlugins"));
const buildInterfaceContext_1 = __importDefault(require("./buildInterfaceContext"));
const buildFieldTypeContext_1 = __importDefault(require("./buildFieldTypeContext"));
const buildSequelizeModels_1 = __importDefault(require("./buildSequelizeModels"));
const buildServices_1 = __importDefault(require("./buildServices"));
const buildRootQueries_1 = __importDefault(require("./buildRootQueries"));
const buildRootMutations_1 = __importDefault(require("./buildRootMutations"));
const buildRootSubscription_1 = __importDefault(require("./buildRootSubscription"));
function buildGraphQLContext(sequelize, config, buildOptions) {
    const plugins = (0, buildPlugins_1.default)(config.plugins || []);
    const sgContext = {
        sequelize: sequelize,
        schemas: (0, applyPluginsToSchemas_1.default)(config.schemas || [], plugins, config, buildOptions),
        models: {},
        services: {},
        typeConfig: (typeName) => null
    };
    const resolveContext = (0, buildResolverContext_1.default)(config.hooks || [], sgContext);
    const interfaceContext = (0, buildInterfaceContext_1.default)(sgContext);
    const context = {
        hookFieldResolve: (name, options) => resolveContext.hookFieldResolve(name, options),
        hookQueryResolve: (name, options) => resolveContext.hookQueryResolve(name, options),
        hookSubscriptionResolve: (name, options) => resolveContext.hookSubscriptionResolve(name, options),
        hookMutationResolve: (name, options) => resolveContext.hookMutationResolve(name, options),
        interface: (str) => {
            return interfaceContext.interface(str);
        },
        registerInterface: (name, gInterface) => {
            return interfaceContext.registerInterface(name, gInterface);
        },
        typeConfig: (typeName) => null
    };
    const fieldTypeContext = (0, buildFieldTypeContext_1.default)(config.types || [], config.dataTypes || [], config.schemas || [], context);
    context.typeConfig = (typeName) => fieldTypeContext.typeConfig(typeName);
    sgContext.typeConfig = (typeName) => fieldTypeContext.typeConfig(typeName);
    sgContext.models = (0, applyPluginsToModels_1.default)((0, buildSequelizeModels_1.default)(sequelize, (config.schemas || []).filter((s) => {
        return s instanceof __1.SGSchema;
    }), sgContext), plugins, buildOptions.defaultPlugin || {});
    sgContext.services = (0, buildServices_1.default)(config.services || [], sgContext);
    const rootQueries = (0, buildRootQueries_1.default)([
        ...(config.schemas || []).map((schema) => schema.config.queries),
        config.queries || {}
    ], context, buildOptions.nodeQueryConfig);
    const payloadFields = {};
    const rootQueryObject = new graphql_1.GraphQLObjectType({
        name: 'RootQuery',
        fields: () => rootQueries
    });
    const schemaConfig = { query: rootQueryObject };
    if (lodash_1.default.keys(rootQueries).length > 0) {
        payloadFields.relay = {
            description: 'Hack to workaround https://github.com/facebook/relay/issues/112 re-exposing the root query object',
            type: new graphql_1.GraphQLNonNull(rootQueryObject),
            resolve: () => {
                return {};
            }
        };
    }
    const rootMutations = (0, buildRootMutations_1.default)([
        ...(config.schemas || []).map((schema) => schema.config.mutations),
        config.mutations || {}
    ], payloadFields, context);
    if (lodash_1.default.keys(rootMutations).length > 0) {
        schemaConfig.mutation = new graphql_1.GraphQLObjectType({
            name: 'RootMutation',
            fields: () => rootMutations
        });
    }
    const rootSubscriptions = (0, buildRootSubscription_1.default)([
        ...(config.schemas || []).map((schema) => schema.config.subscriptions),
        config.subscriptions || {}
    ], context);
    if (lodash_1.default.keys(rootSubscriptions).length > 0) {
        schemaConfig.subscription = new graphql_1.GraphQLObjectType({
            name: 'RootSubscription',
            fields: () => rootSubscriptions
        });
    }
    return {
        sgContext: sgContext,
        graphQLSchema: new graphql_1.GraphQLSchema(schemaConfig)
    };
}
exports.buildGraphQLContext = buildGraphQLContext;
//# sourceMappingURL=index.js.map