"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseSGSchema = void 0;
class BaseSGSchema {
    constructor(name, options = {}) {
        this.name = name;
        this.config = {
            fields: {},
            links: {},
            queries: {},
            subscriptions: {},
            mutations: {},
            methods: {},
            statics: {}
        };
        this.options = options;
    }
    /**
     * Add the model base fields, and each field has a corresponding database column.
     * In default, each field generate a GraphQL field, unless it config with "hidden:true".
     */
    fields(fields) {
        this.config.fields = Object.assign(this.config.fields, fields);
        return this;
    }
    /**
     * Add the model link fields, and each link generate a GraphQL field but no corresponding database column.
     */
    links(links) {
        this.config.links = Object.assign(this.config.links, links);
        return this;
    }
    /**
     * Add the GraphQL query methods.
     */
    queries(queries) {
        // TODO duplicate check
        this.config.queries = Object.assign(this.config.queries, queries);
        return this;
    }
    /**
     * Add the GraphQL subscription methods.
     */
    subscriptions(subscriptions) {
        // TODO duplicate check
        this.config.subscriptions = Object.assign(this.config.subscriptions, subscriptions);
        return this;
    }
    /**
     * Add the GraphQL mutataion methods.
     */
    mutations(mutations) {
        // TODO duplicate check
        this.config.mutations = Object.assign(this.config.mutations, mutations);
        return this;
    }
    /**
     * Add instance method to current Schema.
     */
    methods(methods) {
        this.config.methods = Object.assign(this.config.methods, methods);
        return this;
    }
    /**
     * Add statics method to current Schema.
     */
    statics(statics) {
        this.config.statics = Object.assign(this.config.statics, statics);
        return this;
    }
    plugin(plugin, options) {
        plugin(this, options);
    }
}
exports.BaseSGSchema = BaseSGSchema;
//# sourceMappingURL=BaseSGSchema.js.map